
/*
 * Biblioteca para ler informacoes do /proc
 * Marcelo Veiga Neves
 * veiga@inf.ufsm.br
 * Tue Mar 2 00:33:5 2004
 */

#ifndef _PROC_H_
#define _PROC_H_

#include <time.h>

/* arquivos do /proc usados */
#define STAT       "/proc/stat"
#define CPUINFO    "/proc/cpuinfo"
#define LOADAVG    "/proc/loadavg"
#define OSRELEASE  "/proc/sys/kernel/osrelease"
#define HOSTNAME   "/proc/sys/kernel/hostname"
#define DOMAINNAME "/proc/sys/kernel/domainname"
#define OSTYPE     "/proc/sys/kernel/ostype"
#define PANIC      "/proc/sys/kernel/panic"
#define THREADMAX  "/proc/sys/kernel/threads-max"
#define FILE_MAX   "/proc/sys/fs/file-max"
#define FILE_NR    "/proc/sys/fs/file-nr"
#define INODE_NR   "/proc/sys/fs/inode-nr"
#define MEMINFO    "/proc/meminfo"
#define NET_DEV    "/proc/net/dev"
#define CMDLINE    "/proc/cmdline"
#define UPTIME     "/proc/uptime"
#define NET_UDP    "/proc/net/udp"
#define NET_TCP    "/proc/net/tcp"
#define MOUNTS     "/proc/mounts"

/* outros arquivos usados */
#define UTMP       "/var/run/utmp"

/* Atencao: dependendo da configurao do kernel, pode ser necessario ter previlegio de
   root para poder ler alguns desses arquivos */

/* o tempo default  1 segundo em microsegundos*/
#define DEFAULT_TIME 1000000

/* tempo utilizado pelas funcoes _proc_cpu_status() e _proc_cpu_pid() */
#define _SLEEP 1

/* Essa funcao serve para inicilizar os intervalos de leitura dos arquivos,
 * se nenhum valor for setado a biblioteca assumira DEFAULT_TIME como default 
 */
void proc_set_interval(double interval);


/* retorna o numero de segundos que o kernel espera antes de reinicaiar em um 
 * kernel panic. Se este valor for 0, significa que o "auto reboot" apos um 
 * kernel panic esta desabilitado. A funcao retorna -1 em caso erro.
 * A funcao pode setar o errno dos erros de fopen, fscanf e fclose.
 */
int proc_panic(void);


/* retorna o numero mximo de threads que pode ser utilizado pelo kernel. 
 * A funcao retorna -1 em caso de erro.
 */
int proc_threads_max(void);


/* preenche o string s com o hostname. A funcao retorna o numero de caracteres
 * lidos ou 0 em caso de erro 
 */
int proc_hostname(char *s);


/* preenche o string s com o domainname. A funcao retorna o numero de caracteres
 * lidos ou 0 em caso de erro 
 */
int proc_domainname(char *s);


/* preenche o string s com o tipo do sistema operacinal (ex: Linux). A funcao
 * retorna o numero de caracteres lidos ou 0 em caso de erro 
 */
int proc_ostype(char *s);


/* preenche o string s com os argumentos da linha de comando do kernel. A funcao
 * retorna o numero de caracteres lidos ou 0 em caso de erro 
 */
int proc_cmdline(char *s);


/* preenche o string s com a versao do kernel. A funcao retorna o numero de
 * caracteres lidos ou 0 em caso de erro.
 */
int proc_osrelease(char *s);


/* uptime */
typedef struct {
    double last;
    double uptime;		/* em segundos */
    double uptime_idle;
} uptime_t;


/* retorna o uptime (tempo que passou desde a inicializacao da maquina) em segundos 
 * A funcao retorna -1 em caso de erro.
 */
double proc_uptime(void);


/* retorna o uptime_idle (tempo que a maquina ficou ociosa dede a inicializacao) em segundos 
 * A funcao retorna -1 em caso de erro.
 */
double proc_uptime_idle(void);


/* descritores de arquivo */
typedef struct {
    double last;
    int system;			/*  */
    int disp;			/* numero de fd disponiveis no momento */
    int limit;			/* numero limite de fd */
} fd_t;


/* retorna o numero maximo de descritores de arquivos que o kernel pode alocar. 
 * A funcao retorna -1 em caso de erro.
 */
int proc_fd_max(void);


/* retorna o numero de descritores de arquivos reservados para o sistema (alocados).
 * A funcao retorna -1 em caso de erro.
 */
int proc_fd_system(void);


/* retorna o numero de descritores de arquivos disponiveis no momento.
 * A funcao retorna -1 em caso de erro.
 */
int proc_fd_disp(void);


/* retorna o numero maximo de descritores de arquivos (limite).
 * A funcao retorna -1 em caso de erro.
 */
int proc_fd_limit(void);


/* inodes */
typedef struct {
    double last;
    int alloc;			/* nr_inodes */
    int free;			/* nr_inodes_free */
} inode_t;


/* retorna o numero de inodes alocados pelo sistema (nr_inodes). 
 * A funcao retorna -1 em caso de erro. 
 */
int proc_inode_alloc(void);


/* retorna o numero de inodes livres (nr_free_inodes). 
 * A funcao retorna -1 em caso de erro. 
 */
int proc_inode_free(void);


/* retorna o numero de inodes em uso (nr_inodes - nr_free_inodes)
 * A funcao retorna -1 em caso de erro.
 */
int proc_inode_inuse(void);


/* carga media do sistema */
typedef struct {
    double last;
    float one;
    float five;
    float fifteen;
    int running;
    int total;
    int last_id;
} load_t;


/* retorna a carga media do sistema no ultimo minuto (load_one)
 * A funcao retorna -1 em caso de erro.
 */
float proc_load_one(void);


/* retorna a carga media do sistema nos ultimos 5 minutos (load_five)
 * A funcao retorna -1 em caso de erro.
 */
float proc_load_five(void);


/* retorna a carga media do sistema nos ultimos 15 minutos (load_fifteen)
 * A funcao retorna -1 em caso de erro.
 */
float proc_load_fifteen(void);


/* retorna o numero de processos em execucao nesse instante.
 * A funcao retorna -1 em caso de erro.
 */
int proc_process_running(void);


/* retorna o numero total de processos existentes nesse instante.
 * A funcao retorna -1 em caso de erro.
 */
int proc_process_total(void);


/* retorna o numero do ultimo PID (process ID) usado.
 * A funcao retorna -1 em caso de erro.
 */
int proc_process_last(void);


/* memoria */
typedef struct {
    double last;
    int mem_total;		/*memoria total em kbytes */
    int mem_used;
    int mem_free;
    int mem_shared;
    int buffers;
    int cached;
    int active;
    int inactive;
    int high_total;
    int high_free;
    int low_total;
    int low_free;
    int swap_total;		/*memoria total em kbytes */
    int swap_free;
    int swap_used;
    int swap_cached;
    double mem_pused;		/* porcentagem de uso de memoria */
    double swap_pused;		/* porcentagem de uso de swap */
} meminfo_t;


/* retorna a quantidade total de memoria fisica RAM (em kilobytes).
 * A funcao retorna -1 em caso de erro.
 */
int proc_mem_total(void);


/* retorna a quantidade de memoria fisica RAM livre (em kilobytes).
 * A funcao retorna -1 em caso de erro.
 */
int proc_mem_free(void);


/* retorna a quantidade de memoria fisica RAM usada (em kilobytes).
 * A funcao retorna -1 em caso de erro.
 */
int proc_mem_used(void);


/* retorna a quantidade de memoria fisica RAM compartilhada (em kilobytes).
 * (Obs: essa opcao nao eh usada por alguns kernels pra manter compatibilidade).
 * A funcao retorna -1 em caso de erro.
 */
int proc_mem_shared(void);


/* retorna a quantidade de memoria fisica RAM usada por arquivos de buffer 
 * (em kilobytes). A funcao retorna -1 em caso de erro.
 */
int proc_mem_buffers(void);


/* retorna a quantidade de memoria fisica RAM usada como memoria cache. 
 * (em kilobytes). A funcao retorna -1 em caso de erro.
 */
int proc_mem_cached(void);


/* retorna a quantidade de memoria em buffer ou paginas de memoria cache que esta
 * em uso ativo (em kilobytes). A funcao retorna -1 em caso de erro.
 */
int proc_mem_active(void);


/* deve ser o contrario da anterior?? :) 
 * A funcao retorna -1 em caso de erro.
 */
int proc_mem_inactive(void);


/* retorna a quantidade de memoria que nao  mapeada diretamente no espaco do
 * kernel (em kilobytes). A funcao retorna -1 em caso de erro.
 */
int proc_mem_high_total(void);


/* retorna a quantidade de memoria livre que nao  mapeada diretamente no espaco do
 * kernel (em kilobytes). A funcao retorna -1 em caso de erro.
 */
int proc_mem_high_free(void);


/* retorna a quantidade de memoria que  mapeada diretamente no espaco do
 * kernel (em kilobytes). A funcao retorna -1 em caso de erro.
 */
int proc_mem_low_total(void);


/* retorna a quantidade de memoria livre que  mapeada diretamente no espaco do
 * kernel (em kilobytes). A funcao retorna -1 em caso de erro.
 */
int proc_mem_low_free(void);


/* retorna o quantidade total de swap (em kilobytes).
 * A funcao retorna -1 em caso de erro.
 */
int proc_swap_total(void);


/* retorna o quantidade de swap livre (em kilobytes).
 * A funcao retorna -1 em caso de erro.
 */
int proc_swap_free(void);


/* retorna o quantidade de swap usada (em kilobytes).
 * A funcao retorna -1 em caso de erro.
 */
int proc_swap_used(void);


/* retorna o quantidade de swap usada como memoria cache (em kilobytes).
 * A funcao retorna -1 em caso de erro.
 */
int proc_swap_cached(void);


/* retorna a porcentagem de memoria utilizada.
 * A funcao retorna -1 em caso de erro.
 */
double proc_mem_pused(void);


/* retorna a porcentagem de swap utilizada.
 * A funcao retorna -1 em caso de erro.
 */
double proc_swap_pused(void);


/* retorna o pid do processo de nome exe. Se tiver mais de um processo com esse
 * nome, a funcao vai retornar o de maior PID.
 * A funcao retorna -1 em caso de erro.
 */
int proc_getpid(char *exe);

/* retorna o tempo da maquina, em segundos, desde "the Epoch" (01/01/1970)                 
 * A funcao retorna -1 em caso de erro.
 */
time_t proc_time(void);


/* porcentagem e cpu jiffies */
typedef struct {
    double last;
    int u;			/* cpu jiffies */
    int n;
    int s;
    int i;
    double user;		/* porcentagem */
    double nice;
    double system;
    double idle;
} cpu_t;

/* stats */
typedef struct {
    double last;
    int cpu_num;
    int page_in;
    int page_out;
    int swap_in;
    int swap_out;
    int intr;
    int ctxt;
    int btime;
} stat_t;

/* retorna o numero de CPUs da maquina.
 * A funcao retorna -1 em caso de erro.
 */
int proc_cpu_num(void);

/* retorna o numero de jiffies em modo user.
 * A funcao retorna -1 em caso de erro.
 */
int proc_jiffies_user(void);


/* retorna o numero de jiffies em modo nice.
 * A funcao retorna -1 em caso de erro.
 */
int proc_jiffies_nice(void);


/* retorna o numero de jiffies em modo system.
 * A funcao retorna -1 em caso de erro.
 */
int proc_jiffies_system(void);


/* retorna o numero de jiffies em modo idle.
 * A funcao retorna -1 em caso de erro.
 */
int proc_jiffies_idle(void);


/* retorna a porcentagem de tempo de cpu gasto por usuarios
 * A funcao retorna -1 em caso de erro.
 */
double proc_cpu_user(void);


/* retorna a porcentagem de tempo de cpu nice
 * A funcao retorna -1 em caso de erro.
 */
double proc_cpu_nice(void);


/* retorna a porcentagem de tempo de cpu gasto pelo sistema
 * A funcao retorna -1 em caso de erro.
 */
double proc_cpu_system(void);


/* retorna a porcentagem de tempo de cpu ociosa
 * A funcao retorna -1 em caso de erro.
 */
double proc_cpu_idle(void);


/* retorna o nmero de pginas de memoria que entraram no sistema.
 * A funcao retorna -1 em caso de erro.
 */
int proc_page_in(void);


/* retorna o nmero de pginas de memoria que sairam (pro disco),
 * A funcao retorna -1 em caso de erro.
 */
int proc_page_out(void);


/* retorna o nmero de pginas de swap trazidas pelo sistema,
 * A funcao retorna -1 em caso de erro.
 */
int proc_swap_in(void);


/* retorna o nmero de pginas de swap que voltaram (pro disco) ?
 * A funcao retorna -1 em caso de erro.
 */
int proc_swap_out(void);


/* retorna o nmero de interrupes recebidas a partir da inicializao
 * do sistema. 
 * A funcao retorna -1 em caso de erro.
 */
int proc_intr(void);


/* retorna nmero de mudanas de contexto que o sistema realizou 
 * (desde a inicializacao?)
 * A funcao retorna -1 em caso de erro.
 */
int proc_ctxt(void);


/* retorna o tempo de inicializacao (boot time), isto eh, um timestamp 
 * da hora em que o sistema foi inicializado.
 * A funcao retorna -1 em caso de erro.
 */
int proc_btime(void);


#define TAMMAX 60
#define NUMMAX 2

/* informacoes da CPU */

typedef struct {
    char vendor_id[TAMMAX];
    int family;
    int model;
    char model_name[TAMMAX];
    float mhz;			/* cpu MHz */
    int cache_size;		/* em KB */
    float bogomips;
} _cpuinfo_t;


typedef struct {
    double last;
    int n_cpu;
    _cpuinfo_t cpu[NUMMAX];
} cpuinfo_t;


/* retorna um identificador da familia do processador. id representa a
 * numero do processador, se a maquina for monoprocessada use id = 0.
 * A funcao retorna -1 em caso de erro.
 */
int proc_cpu_family(int id);


/* retorna um identificador do modelo do processador. id representa a
 * numero do processador, se a maquina for monoprocessada use id = 0.
 * A funcao retorna -1 em caso de erro.
 */
int proc_cpu_model(int id);


/* retorna o tamanho da cache do processador (em KB) identificado por 
 * id, se a maquina for monoprocessada use id = 0.
 * A funcao retorna -1 em caso de erro.
 */
int proc_cpu_cache_size(int id);


/* retorna um a frequencia da CPU (em MHz) identificada por id, 
 * se a maquina for monoprocessada use id = 0.
 * A funcao retorna -1 em caso de erro.
 */
float proc_cpu_mhz(int id);


/* retorna retorna o bogomips da CPU identificada por id, 
 * se a maquina for monoprocessada use id = 0. 
 * A funcao retorna -1 em caso de erro.
 */
float proc_cpu_bogomips(int id);


/* preenche uma string s com o nome do fabricante do processador 
 * identificado por id, se a maquina for monoprocessada use id = 0.
 * A funcao retorna 0 em caso de erro ou 1 para sucesso.
 */
int proc_cpu_vendor_id(int id, char *s);


/* preenche uma string s com o nome do modelo do processador
 * identificado por id, se a maquina for monoprocessada use id = 0.
 * A funcao retorna 0 em caso de erro ou 1 para sucesso.
 */
int proc_cpu_model_name(int id, char *s);


/* interfaces de rede */
typedef struct {
    int id;			/* identidica eth0, eth1, etc */

    /* Receive */
    int cbytes_in;		/* valor acumulado */
    int cpkts_in;		/* valor acumulado */
    double bytes_in;            /* em bytes/s */		
    double pkts_in;		/* em packets/s */
    int errs_in;
    int drop_in;
    int fifo_in;
    int frame;
    int compressed_in;
    int multicast;

    /* Transmit */
    int cbytes_out;
    int cpkts_out;
    double bytes_out;
    double pkts_out;
    int errs_out;
    int drop_out;
    int fifo_out;
    int colls;
    int carrier;
    int compressed_out;

} _eth_t;


#define NUMMAXETH 2		/* numero maximo de interfaces de rede */

/* rede */
typedef struct {
    double last;		/* ultima leitura */
    int n_eth;			/* quantidade de interfaces de rede */
    _eth_t eth[NUMMAXETH];	/* vetor de interfaces */
} net_t;

/* retorna o numero de interfaces de rede que a maquina possui.
 * A funcao retorna -1 em caso de erro.
 */
int proc_eth_num(void);


/* retorna a quantidade de bytes por segundo que a interface identificada
 * por eth esta recebendo.
 * A funcao retorna -1 em caso de erro.
 */
double proc_bytes_in(int eth);


/* retorna a quantidade de bytes que a interface identificada por eth jah recebeu.
 * A funcao retorna -1 em caso de erro.
 */
int proc_cbytes_in(int eth);


/* retorna o numero de pacotes por segundo que a interface identificada
 * por eth esta recebendo.
 * A funcao retorna -1 em caso de erro.
 */
double proc_pkts_in(int eth);


/* retorna o numero de pacotes que a interface identificada por eth jah recebeu.
 * A funcao retorna -1 em caso de erro.
 */
int proc_cpkts_in(int eth);


/* retorna a quantidade de erros de receive que o drive do dispositivo
 * identificado por eth ja detectou.
 * A funcao retorna -1 em caso de erro.
 */
int proc_errs_in(int eth);


/* retorna o numero de pacotes dropped no recebimento pelo drive do dispositivo
 * identificado por eth ja detectou.
 * A funcao retorna -1 em caso de erro.
 */
int proc_drop_in(int eth);


/* retorna o numero de erros em buffer FIFO no recebimento que drive do
 * dispositivo identificado por eth ja detectou.
 * A funcao retorna -1 em caso de erro.
 */
int proc_fifo_in(int eth);


/* retorna o numero de erros de "packet framing" que o drive do
 * dispositivo identificado por eth ja detectou.
 * A funcao retorna -1 em caso de erro.
 */
int proc_frame(int eth);


/* retorna o numero de pacotes compressed recebidos pelo drive do dispositivo
 * identificado por eth.
 * A funcao retorna -1 em caso de erro.
 */
int proc_compressed_in(int eth);


/* retorna o numero de "multicast frames" recebidos ou transmitidos pelo drive
 * do dispositivo identificado por eth.
 * A funcao retorna -1 em caso de erro.
 */
int proc_multicast(int eth);


/* retorna a quantidade de bytes por segundo que a interface identificada
 * por eth esta transmitindo.
 * A funcao retorna -1 em caso de erro.
 */
double proc_bytes_out(int eth);


/* retorna a quantidade de bytes que a interface identificada por eth 
 * jah transmitiu.
 * A funcao retorna -1 em caso de erro.
 */
int proc_cbytes_out(int eth);


/* retorna o numero de pacotes por segundo que a interface identificada
 * por eth esta transmitindo.
 * A funcao retorna -1 em caso de erro.
 */
double proc_pkts_out(int eth);


/* retorna o numero de pacotes que a interface identificada por eth jah transmitiu.
 * A funcao retorna -1 em caso de erro.
 */
int proc_cpkts_out(int eth);


/* retorna a quantidade de erros de transmit que o drive do dispositivo
 * identificado por eth ja detectou.
 * A funcao retorna -1 em caso de erro.
 */
int proc_errs_out(int eth);


/* retorna o numero de pacotes dropped na transmissao pelo drive do dispositivo
 * identificado por eth.
 * A funcao retorna -1 em caso de erro.
 */
int proc_drop_out(int eth);


/* retorna o numero de erros em buffer FIFO na transmissao que drive do
 * dispositivo identificado por eth ja detectou.
 * A funcao retorna -1 em caso de erro.
 */
int proc_fifo_out(int eth);


/* retorna o numero de colisoes detectadas na interface identificada por eth.
 * A funcao retorna -1 em caso de erro.
 */
int proc_colls(int eth);


/* retorna o numero de "carrier losses" que drive do dispositivo identificado
 * por eth ja detectou.
 * A funcao retorna -1 em caso de erro.
 */
int proc_carrier(int eth);


/* retorna o numero de pacotes compressed transmitidos pelo drive do dispositivo
 * identificado por eth.
 * A funcao retorna -1 em caso de erro.
 */
int proc_compressed_out(int eth);



/* O conjunto de funcoes abaixo eh de uso interno a biblioteca.
 * essas funcoes servem para ler os arquivos e preencher as estruturas que so 
 * acessadas pelas funoes proc_*(). Essa abordagem garante a integridades dos 
 * dados j que as funes proc_*() sempre testam o "tempo de vida" das mtricas.
 */
int _proc_read_integer(char *filename);
int _proc_read_string(char *filename, char *s);
int _proc_read_line(char *filename, char *s);
double _proc_randon_interval(int min, int max);
int _proc_read_uptime(uptime_t * up);
int _proc_read_fd(fd_t * fd);
int _proc_read_inode(inode_t * inode);
int _proc_read_load(load_t * load);
int _proc_read_meminfo(meminfo_t * meminfo);
void _proc_clean_name(char *process);
int _proc_read_stat(stat_t * stat);
int _proc_read_cpu(cpu_t * cpu);
int _proc_cpu_status(cpu_t * cpu);
int _proc_read_cpuinfo(cpuinfo_t * cpuinfo);
int _proc_read_net(net_t * net, int mode);
int _proc_read_link(char *path, char *s);
double _proc_now(void); /* retorna o tempo em microsegundos */


/* As funcoes abaixo leem informacoes do /proc/PID */


/* retorna 1 se o PID existir ou 0 se nao existir
 */
int proc_pid_exists(int pid);


/* preenche uma string com a linha de comando que lancou o processo 
 * identificado por pid.
 * A funcao retorna o numero de caracteres da string ou 0 em caso de erro.
 */
int proc_cmdline_pid(int pid, char *s);

/* preenche uma string com o nome do usurio que lancou o processo 
 * identificado por pid.
 * A funcao retorna o numero de caracteres da string ou 0 em caso de erro.
 */
int proc_username_pid(int pid, char *s);

/* preenche uma string com o nome e caminho completo do arquivo executavel.
 * A funcao retorna o numero de caracteres da string ou 0 em caso de erro.
 */
int proc_exe_pid(int pid, char *s);

/* preenche uma string com o formato do arquivo executavel (ex: elf).
 * A funcao retorna o numero de caracteres da string ou 0 em caso de erro.
 */
int proc_binfmt_pid(int pid, char *s);

/* preenche uma string com o diretorio de trabalho do processo (cwd - 
 * current working directory).
 * A funcao retorna o numero de caracteres da string ou 0 em caso de erro.
 */
int proc_cwd_pid(int pid, char *s);


/* preenche uma string com o diretorio raiz do processo (root).
 * A funcao retorna o numero de caracteres da string ou 0 em caso de erro.
 */
int proc_root_pid(int pid, char *s);


#define FDMAX 10

/* descritores de arquivos usados pelo processo */
typedef struct {
    double last;
    int num;			/* numero de fd usados pelo processo */
    char files[FDMAX][TAMMAX];	/* vetor de nomes de arquivos abertos pelo processo */
} fd_pid_t;


/* retorna o numero de descritores de arquivos utilizados pelo processo
 * identificado por pid.
 * A funcao retorna o numero de caracteres da string ou 0 em caso de erro.
 */
int proc_fd_num_pid(int pid);


/* preenche uma string com o nome do arquivo que possui um descritor 
 *identificado por id.
 * A funcao retorna o numero de caracteres da string ou 0 em caso de erro.
 */
int proc_fd_file_pid(int pid, int id, char *s);


typedef struct {
    double last;
    int size;			/* total program size */
    int resident;		/* size of memory portions */
    int shared;			/* number of pages that are shared */
    int trs;			/* number of pages that are 'code' */
    int drs;			/*  number of pages of data/stack */
    int lrs;			/* number of pages of library */
    int dt;			/* number of dirty pages */
} statm_pid_t;


/* retorna o tamanho total do programa cujo processo  identificado por pid 
 * A funcao retorna -1 em caso de erro.
 */
int proc_statm_size_pid(int pid);


/* retorna o tamanho das porcoes de memoria (memory portions) do processo
 * identificado por pid. 
 * A funcao retorna -1 em caso de erro.
 */
int proc_statm_resident_pid(int pid);


/* retorna o numero de paginas que sao compartilhadas do processo 
 * identificado por pid. 
 * A funcao retorna -1 em caso de erro.
 */
int proc_statm_shared_pid(int pid);


/* retorna o numero de paginas que sao de "code" do processo 
 * identificado por pid. 
 * A funcao retorna -1 em caso de erro.
 */
int proc_statm_trs_pid(int pid);


/* retorna o numero de paginas que sao de dados (data/stack) do processo 
 * identificado por pid. 
 * A funcao retorna -1 em caso de erro.
 */
int proc_statm_drs_pid(int pid);


/* retorna o numero de paginas que sao de biblioteca (library) do processo 
 * identificado por pid. 
 * A funcao retorna -1 em caso de erro.
 */
int proc_statm_lrs_pid(int pid);


/* retorna o numero de "dirty pages" do processo identificado por pid. 
 * A funcao retorna -1 em caso de erro.
 */
int proc_statm_dt_pid(int pid);


/* informacoes do processo */
typedef struct {
    double last;
    int pid;
    char exe[TAMMAX];
    char state;			/* R, S, D, Z, T ou W */
    int ppid;
    int pgrp;
    int sid;
    int terminal;
    int ptgid;

    /* faults */
    unsigned long flags;
    unsigned long min_flt;
    unsigned long cmin_flt;
    unsigned long maj_flt;
    unsigned long cmaj_flt;

    /* cpu jiffies */
    long int utime;
    long int stime;
    long int cutime;
    long int cstime;
    long int counter;		/* numero maximo de jiffies */
    long int priority;
    unsigned long timeout;
    unsigned long it_real_value;
    long int start_time;

    /* memory */
    unsigned vsize;		/* em bytes */
    double pused;		/* % em relacao ao total de memoria usada */
    unsigned rss;
    unsigned rss_rlim;

    /* addr */
    unsigned long start_code;
    unsigned long end_code;
    unsigned long start_stack;
    unsigned long kstk_esp;
    unsigned long kstk_eip;
} stat_pid_t;

typedef struct {
    long last;
    double puser;		/* porcentagem */
    double pcuser;
    double psystem;
    double pcsystem;
    long int utime;
    long int stime;
    long int cutime;
    long int cstime;
} cpu_pid_t;

/* retorna um char com o estado do processo identificado por pid.
 *
 * - o caracter "state" pode ser:
 * R = running
 * S = interruptible sleeping or swapping 
 * D = uninterruptible sleeping or swapping
 * Z = zonbie
 * T = traced or stopped
 * W = swapped
 *
 * A funcao retorna -1 em caso de erro.
 */
char proc_state_pid(int pid);


/* preenche uma string como o nome do programa (s nome, sem caminho) cujo
 * do processo  identificado por pid 
 * A funcao retorna -1 em caso de erro.
 */
int proc_exe_name_pid(int pid, char *s);


/* retorna o ? do processo  identificado por pid 
 * A funcao retorna -1 em caso de erro.
 */
int proc_statm_size_pid(int pid);


/* retorna o ? do processo  identificado por pid 
 * A funcao retorna -1 em caso de erro.
 */
int proc_ppid_pid(int pid);


/* retorna o ? do processo  identificado por pid 
 * A funcao retorna -1 em caso de erro.
 */
int proc_pgrp_pid(int pid);


/* retorna o  do processo  identificado por pid 
 * A funcao retorna -1 em caso de erro.
 */
int proc_sid_pid(int pid);


/* retorna o ? do processo  identificado por pid 
 * A funcao retorna -1 em caso de erro.
 */
int proc_terminal_pid(int pid);


/* retorna o ? do processo  identificado por pid 
 * A funcao retorna -1 em caso de erro.
 */
int proc_ptgid_pid(int pid);


/* retorna o ? do processo  identificado por pid 
 * A funcao retorna -1 em caso de erro.
 */
unsigned long proc_flags_pid(int pid);


/* retorna o ? do processo  identificado por pid 
 * A funcao retorna -1 em caso de erro.
 */
unsigned long proc_min_flt_pid(int pid);


/* retorna o ? do processo  identificado por pid 
 * A funcao retorna -1 em caso de erro.
 */
unsigned long proc_cmin_flt_pid(int pid);


/* retorna o ? do processo  identificado por pid 
 * A funcao retorna -1 em caso de erro.
 */
unsigned long proc_maj_flt_pid(int pid);


/* retorna o ? do processo  identificado por pid 
 * A funcao retorna -1 em caso de erro.
 */
unsigned long proc_cmaj_flt_pid(int pid);


/* retorna o  do processo  identificado por pid 
 * A funcao retorna -1 em caso de erro.
 */
long int proc_utime_pid(int pid);


/* retorna o ? do processo  identificado por pid 
 * A funcao retorna -1 em caso de erro.
 */
long int proc_stime_pid(int pid);


/* retorna o ? do processo  identificado por pid 
 * A funcao retorna -1 em caso de erro.
 */
long int proc_cutime_pid(int pid);


/* retorna o ? do processo  identificado por pid 
 * A funcao retorna -1 em caso de erro.
 */
long int proc_cstime_pid(int pid);


/* retorna o ? do processo  identificado por pid 
 * A funcao retorna -1 em caso de erro.
 */
long int proc_counter_pid(int pid);


/* retorna o ? do processo  identificado por pid 
 * A funcao retorna -1 em caso de erro.
 */
long int proc_priority_pid(int pid);


/* retorna o ? do processo  identificado por pid 
 * A funcao retorna -1 em caso de erro.
 */
unsigned long proc_timeout_pid(int pid);


/* retorna o ? do processo  identificado por pid 
 * A funcao retorna -1 em caso de erro.
 */
unsigned long proc_it_real_value_pid(int pid);


/* retorna o ? do processo  identificado por pid 
 * A funcao retorna -1 em caso de erro.
 */
long int proc_start_time_pid(int pid);


/* retorna o ? do processo  identificado por pid 
 * A funcao retorna -1 em caso de erro.
 */
unsigned proc_vsize_pid(int pid);


/* retorna o ? do processo  identificado por pid 
 * A funcao retorna -1 em caso de erro.
 */
double proc_mem_pused_pid(int pid);


/* retorna o ? do processo  identificado por pid 
 * A funcao retorna -1 em caso de erro.
 */
unsigned proc_rss_pid(int pid);


/* retorna o  do processo  identificado por pid 
 * A funcao retorna -1 em caso de erro.
 */
unsigned proc_rss_rlim_pid(int pid);


/* retorna o ? do processo  identificado por pid 
 * A funcao retorna -1 em caso de erro.
 */
unsigned long proc_start_code_pid(int pid);


/* retorna o ? do processo  identificado por pid 
 * A funcao retorna -1 em caso de erro.
 */
unsigned long proc_end_code_pid(int pid);


/* retorna o ? do processo  identificado por pid 
 * A funcao retorna -1 em caso de erro.
 */
unsigned long proc_start_stack_pid(int pid);


/* retorna o ? do processo  identificado por pid 
 * A funcao retorna -1 em caso de erro.
 */
unsigned long proc_kstk_esp_pid(int pid);


/* retorna o ? do processo  identificado por pid 
 * A funcao retorna -1 em caso de erro.
 */
unsigned long proc_kstk_eip_pid(int pid);


/* retordo a procentagem do tempo de CPU do usuarios utilizado pelo processo
 * A funcao retorna -1 em caso de erro.
 */
double proc_cpu_user_pid(int pid);


/* retordo a procentagem do tempo de CPU do systema utilizado pelo processo
 * A funcao retorna -1 em caso de erro.
 */
double proc_cpu_system_pid(int pid);


/* retorna o UID do usurio ao qual o processo pertence
 * A funcao retorna -1 em caso de erro.
 */
int proc_uid_pid(int pid);

typedef struct {
	int total;
	int established;
	long last;
} proc_net_con_t;

int _proc_net_connections(proc_net_con_t *net_con, char *file);


/** retorna o numero total de conexoes TCP existentes
 * A funcao retorna -1 em caso de erro.
 */
int proc_tcp_total(void);


/** retorna o numero de conexoes TCP estabelecidas (so as conexoes 
 * que estao sendo utilizadas por programas, ssh, ftp, browser, etc.)
 * A funcao retorna -1 em caso de erro.
 */
int proc_tcp_established(void);


/** retorna o numero total de conexoes UDP existentes
 * A funcao retorna -1 em caso de erro.  
 */
int proc_udp_total(void);


/** retorna o numero de conexoes TCP estabelecidas (so as conexoes
 * que estao sendo utilizadas por programas)
 * A funcao retorna -1 em caso de erro.
 */
int proc_udp_established(void);


/** retorna o numero de usuarios logados na maquina
 * A funcao retorna -1 em caso de erro.
 */
int proc_users(void);


/* entrada da tabela de particoes */
struct mounts_entry {
	char mount[128];     /* diretorio onde a particao esta montada */
	char device[128];    /* disco da particao */
	char type[32];       /* tipo da particao */
	double total;        /* capacidade da particao, em MBytes */
	double free;         /* espaco livres na particao, em MBytes*/
	double pused;        /* percentagem de uso da particao */
	int block_size;      /* tamanho do bloco */
	unsigned int blocks; /* quantidade total de blocos */
	unsigned int bfree;  /* quantidade de blocos livres para superusuario */
	unsigned int bavail; /* quantidade de blocos disponveis aos usuarios */
	unsigned int files;  /* quantidade de file nodes */
	unsigned int ffree;  /* quantidade de file nodes livres*/
};

typedef struct {
	double last;
	int num_parts; /* numero de particoes montadas */
	struct mounts_entry *mounts; /* lista de particoes montadas */
	double total; /* capacidade total dos discos, em MBytes*/
	double free; /* espaco livre nos discos, em MBytes*/
	double pused; /* porcentagem de uso dos discos */
	double part_max_used; /* maior utilizacao de uma particao, em porcentagem */
} proc_disk_t;


/* funcoes de uso interno da biblioteca */
int _proc_stat_fs(char *mount, char *device, struct mounts_entry *part);
int _proc_read_mounts(proc_disk_t *disk);


/* retorna a capacidade total de disco, em MBytes
 * A funcao retorna -1 em caso de erro.
 */
double proc_disk_total();


/* retorna o espaco livre em disco, em MBytes
 * A funcao retorna -1 em caso de erro.
 */
double proc_disk_free();


/* retorna a porcentagem de utilizacao de disco
 * A funcao retorna -1 em caso de erro.
 */
double proc_disk_pused();


/* retorna a porcentagem de utilizacao da particao mais utilizada (mais "cheia")
 * A funcao retorna -1 em caso de erro.
 */
double proc_part_max_used();


/* retorna o numero de particoes montadas
 * A funcao retorna -1 em caso de erro.
 */
int proc_num_parts();


/* preenche uma string (type) com tipo de sistema de arquivos (ex: ext3, vfat, etc.) 
 * da particao que esta montada em "path" (path pode ser, por exemplo, "/home")
 * A funcao retorna -1 em caso de erro, ou 0 para sucesso.
 */
int proc_part_type(const char *path, char *type);


/* preenche uma string (device) com o dispositivo (ex: /dev/hda1) 
 * da particao que esta montada em "path" (path pode ser, por exemplo, "/home")
 * A funcao retorna -1 em caso de erro, ou 0 para sucesso.
 */
int proc_part_device(const char *path, char *device);


/* preenche uma string (mount) com o diretorio (ex: /home) onde 
 * dispositivo "device" (ex: /dev/hda1) esta montado.
 * A funcao retorna -1 em caso de erro, ou 0 para sucesso.
 */
int proc_part_mount(const char *device, char *mount);


/* preenche uma string (path) com uma entrada da tabela de particoes,
 * o inteiro id presenta o indice do vetor de particoes, esse id pode 
 * variar de 0 a num_parts-1
 * A funcao retorna -1 em caso de erro, ou 0 para sucesso.
 */
int proc_get_part(int id, char *path);


/* retorna o tamanho total, em MBytes, da particao montada em "path" (ex: /home)
 * A funcao retorna -1 em caso de erro.
 */
double proc_part_total(const char *path);


/* retorna o espaco livre, em MBytes, na particao montada em "path" (ex: /home)
 * A funcao retorna -1 em caso de erro.
 */
double proc_part_free(const char *path);


/* retorna o porcentagem de utilizacao da particao montada em "path" (ex: /home)
 * A funcao retorna -1 em caso de erro.
 */
double proc_part_pused(const char *path);


/* retorna o tamanho do bloco, em bytes, da particao montada em "path" (ex: /home)
 * A funcao retorna -1 em caso de erro.
 */
int proc_part_block_size(const char *path);


/* retorna a quantidade de blocos da particao montada em "path" (ex: /home)
 * A funcao retorna -1 em caso de erro.
 */
unsigned int proc_part_blocks(const char *path);


/* retorna a quantidade de blocos livres, disponiveis para superusuario,
 * na particao montada em "path" (ex: /home)
 * A funcao retorna -1 em caso de erro.
 */
unsigned int proc_part_bfree(const char *path);


/* retorna a quantidade de blocos livres, disponiveis para usuarios,
 * na particao montada em "path" (ex: /home)
 * A funcao retorna -1 em caso de erro.
 */
unsigned int proc_part_bavail(const char *path);


/* retorna a quantidade file nodes na particao montada em "path" (ex: /home)
 * A funcao retorna -1 em caso de erro.
 */
unsigned int proc_part_files(const char *path);


/* retorna a quantidade file nodes livres na particao montada em "path" (ex: /home)
 * A funcao retorna -1 em caso de erro.
 */
unsigned int proc_part_ffree(const char *path);


#endif				/* _PROC_H_ */
